
/**
 * <p>Title: PSOAgent.java </p>
 * <p>Description: Particle (or agent) of a PSOModel </p>
 * <p>Copyright: Copyright (c) 28/05/2009</p>
 * <p>Institute: School of Computer Science and IT, RMIT University</p>
 * @author Xiaodong Li (xiaodong.li@rmit.edu.au)
 * @version 1.0
 * This is the implementation of the rpso algorithms described in the following paper:
 * Li, X. (2010), "Niching without Niching Parameters: Particle Swarm Optimization using a Ring Topology", 
 * IEEE Transactions on Evolutionary Computation,  Volume 14, No.1, February 2010, pp.150-169.
 * For more information, please refer to: http://goanna.cs.rmit.edu.au/~xiaodong/
 * You need to have RePast 3.1 installed on your machine in order to compile and run the program. 
 * For more informaton on RePast, see: http://repast.sourceforge.net/repast_3/index.html
 */

import java.util.*;
import uchicago.src.sim.gui.*;
import uchicago.src.sim.space.*;
import java.lang.Math.*;


// If an agent, or any other object is to be displayed in a display it
// needs to implement at least the drawable interface which has one method:
// draw(SimGraphics g)
public class PSOAgent implements Drawable {

  // ---------------------------------------------------------------- Members

  /** Two coordindates (x, y) represent this agent on the 2-dimensional lattice **/
  protected int x, y;

  /** An array contains the value for each dimension of the current position of an agent **/
  private double[] present;

  /** An array represents the best location so far of the agent **/
  private double[] pBest;

  /** An array represents the best location so far of the agent **/
  private double[] nBest;
  
  /** An array represents the species seed which is the best position in a species; this is only used for measuring performance**/
  private double[] seed;

  /** The seed fitness value **/
  private double seedFit;

  /** An array represent the current velocity vector of the agent **/
  private double[] v;

  /** The current fitness value **/
  private double fitnessValue;

  /** The best fitness value in history of this agent **/
  private double pBestValue;
  
  /** The best fitness value in history of this agent **/
  private double nBestValue;

  /** The model associated with this agent **/
  private PSOModel model;
    
  /** Agent index for drawing purpose **/
  private int index;
  
  /** Agent index for printing purpose **/
  private int num;
  
  /** Euclidean distance from pBest to the updated present **/
  private double euDist;
  
  /** The sharing fitness value **/
  private double shFit;

  public PSOAgent() {

  }

  public PSOAgent(PSOModel model) {
    // Copy the space and model
    this.model = model;

    // Initalize the arrays
    present = new double[model.getDimension()];
    pBest = new double[model.getDimension()];
    nBest = new double[model.getDimension()];
    seed = new double[model.getDimension()];
    fitnessValue = -1.7976931348623157E308;  
    pBestValue = -1.7976931348623157E308;  
    nBestValue = -1.7976931348623157E308;  
    seedFit = -1.7976931348623157E308;  // changed for maximization
    v = new double[model.getDimension()];     
    index = 0;
    //this.num = num;
    euDist = 0;
    shFit = -1.7976931348623157E308;  
    
    // Obtain right range and left range from PSOParameters object
    double rightRange = model.getRightRange();
    double leftRange = model.getLeftRange();
 
    // Randomly initialize the velocity and pbest
    for (int i=0; i < model.getDimension(); i++) 
    {
         if (model.getFunction().equals("SixHumpCamel") && (i == 1))       
         {   
            rightRange = 1.1; 
            leftRange = -1.1;
         }
         
         present[i] = (rightRange - leftRange) * model.getRandomUniform() + leftRange;
         pBest[i] = present[i];    
         nBest[i] = present[i]; 
         seed[i] = present[i];    
         v[i] = (rightRange - leftRange) * model.getRandomUniform() + leftRange;
       
         // 50% of v[i] are in an opposite direction
         if (model.getRandomUniform() <= 0.5)
            v[i] = -v[i];
    }
  }

 
  /**
   * Set the xy-coordinates of the agent
   * @param x
   * @param y
   */
  public void setXY(int x, int y) {
   this.x = x;
   this.y = y;
 
  }

  /**
   * Return x-coordinate
   * @return
   */
  public int getX() {
    return x;
  }

  /**
   * Return y-coordinate
   * @return
   */
  public int getY() {
    return y;
  }
  
   public double getFitnessValue() {
       //System.out.println("present[0] :"+present[0]+" fitnessValue*** :"+fitnessValue);  
       return fitnessValue;
   }

   public double getPBestValue() {
       return pBestValue;
   }

   public double[] getPresent() {
       return present;
   }
   
   
   public double[] getPBest() {
       return pBest;
   }

   public void setPBest(double[] pb) {
       for (int j=0; j < model.getDimension(); j++)
        pBest[j] = pb[j];
   }

   public void setPBestValue(double pbv) {
       pBestValue = pbv;
   }

   
   public void setNBest(double[] nb) {
       for (int j=0; j < model.getDimension(); j++)
        nBest[j] = nb[j];
   }

   public double getNBestValue() {
       return nBestValue;
   }

   public void setNBestValue(double gbv) {
       nBestValue = gbv;
   }


   public void setSeed(double[] pb, double fit) {
       for (int j=0; j < model.getDimension(); j++)
        seed[j] = pb[j];
       seedFit = fit;
   }
   
   public double[] getSeed() {
       return seed;
   }
  
     
   public double getSeedFit() {
       return seedFit;
   }

   public void setPosition(double[] sp) {
      for (int j=0; j < model.getDimension(); j++)
        present[j] = sp[j];
   }
   
   public void setShFit(double sf) {
       shFit = sf;
   }
   
   public double getShFit() {
       return shFit;
   }
   
   
 
  /**
   * step() is called by the scheduler once per tick.
   */
  public void step() {
 
    // first get the fitness calculated
    fitnessValue = computeFitness(present);
    //System.out.println(num+"\t"+present[0]+"\t"+fitnessValue+"\t"+v[0]);    
    
    // If this is the first iteration, then pbestValue is assigned to the current value
    if (model.getTickCount() == 1) {
        pBestValue = fitnessValue;
        nBestValue = fitnessValue;
    }
    
    // update the pbest
    if (fitnessValue > pBestValue) {
       pBestValue = fitnessValue;
       for (int i=0; i < present.length; i++) 
          pBest[i] = present[i];
    }

//    if (model.getTickCount() < model.getMaxIterations())
        //if ((model.getTickCount()%10 == 0) || (model.getTickCount()==1) || (model.getTickCount()==2) || (model.getTickCount()==5))
/*        if (((model.getTickCount()%5 == 0) || (model.getTickCount()==1)) && model.getBatch() == false) 
        {
            if (model.getDimension() == 1)
            {
                System.out.println(model.getTickCount()+"\t"+present[0]+"\t"+fitnessValue);
                System.out.println(model.getTickCount()+"\t"+pBest[0]+"\t"+pBestValue);      
                System.out.println(model.getTickCount()+"\t"+nBest[0]+"\t"+nBestValue);   
            }
            else
            {
                System.out.println(model.getTickCount()+"\t"+present[0]+"\t"+present[1]);
                System.out.println(model.getTickCount()+"\t"+pBest[0]+"\t"+pBest[1]);      
                System.out.println(model.getTickCount()+"\t"+nBest[0]+"\t"+nBest[1]);  
            }
            System.out.println("");  
        }
*/
 }
 
  /**
   * Draw a line from personal best to current position
   */
  public void positionPlot(int i)
  {
    index = i;
    if (model.getDimension() == 1)
    {
       model.aPlot.plotPoint(present[0], fitnessValue, index);
       model.aPlot.plotPoint(pBest[0], pBestValue, index);   
       // System.out.println("present[0] :"+present[0]+" fitnessValue :"+fitnessValue);            
    }
    else
    {
       model.aPlot.plotPoint(present[0], present[1], index);
       model.aPlot.plotPoint(pBest[0], pBest[1], index);
    }
 
  }
  
  
  /**
   * Draw a point of index and position of the current particle
   */
  public void indexAndPosition(int i)
  {
    index = i;

    model.nichePlot.plotPoint(pBest[0], index, 1001);
    model.nichePlot.plotPoint(nBest[0], index, 1003);
           
/*    if (((model.getTickCount()%5 == 0) || (model.getTickCount()==1)) && model.getBatch() == false)
    {
        System.out.println(model.getTickCount()+"\t"+pBest[0]+"\t"+i+"\t"+pBestValue);       
        System.out.println(model.getTickCount()+"\t"+nBest[0]+"\t"+i+"\t"+nBestValue); 
        
        System.out.println("");
    }
*/      
  }
  
  
  
  /**
   * Draw all personal best and nBest positions (unconnected)
   */
  public void pBestNBestPlot()
  {
    // draw pBest first
    if (model.getDimension() == 1)
       model.aPlot.plotPoint(pBest[0], pBestValue, 1001);
    else
       model.aPlot.plotPoint(pBest[0], pBest[1], 1001);
                
    // then draw nBest
    if (model.getDimension() == 1)
        model.aPlot.plotPoint(nBest[0], nBestValue, 1003);
    else
        model.aPlot.plotPoint(nBest[0], nBest[1], 1003);
 
  }
  
  
  
  protected double computeFitness(double[] present) {

   // Calculate fitness value
   if (model.getFunction().equals("Sphere")) 
       fitnessValue = sphere();
   else if (model.getFunction().equals("Parabola")) 
       fitnessValue = parabola();
   else if (model.getFunction().equals("Himmelblau")) 
       fitnessValue = himmelblau(present);    
   else if (model.getFunction().equals("SixHumpCamelBack")) 
       fitnessValue = sixHumpCamelBack(present); 
   else if (model.getFunction().equals("Shubert2D")) 
       fitnessValue = shubert2d(present);   
   else if (model.getFunction().equals("Shubert3D")) 
       fitnessValue = shubert3d(present);   
   else if (model.getFunction().equals("Shubert4D")) 
       fitnessValue = shubert4d(present);   
   else if (model.getFunction().equals("Foxholes")) 
       fitnessValue = foxholes(present);      
   else if (model.getFunction().equals("Deb5Even")) 
       fitnessValue = deb1(present); 
   else if (model.getFunction().equals("Deb1Even")) 
       fitnessValue = deb2(present);
   else if (model.getFunction().equals("Deb5Uneven")) 
       fitnessValue = deb3(present);   
   else if (model.getFunction().equals("Deb1Uneven")) 
       fitnessValue = deb4(present);          
   else if (model.getFunction().equals("TwoPeakTrap")) 
       fitnessValue = twoPeakTrap(present);          
   else if (model.getFunction().equals("CentralTwoPeakTrap")) 
       fitnessValue = centralTwoPeakTrap(present); 
   else if (model.getFunction().equals("FiveUnevenPeaksTrap")) 
       fitnessValue = fiveUnevenPeaksTrap(present); 
   else if (model.getFunction().equals("Rastrigin")) 
       fitnessValue = rastrigin(present); 
   else if (model.getFunction().equals("Vincent1D")) 
       fitnessValue = vincent1d(present); 
   else if (model.getFunction().equals("Vincent2D")) 
       fitnessValue = vincent2d(present); 
   else if (model.getFunction().equals("Vincent3D")) 
       fitnessValue = vincent3d(present); 
   else if (model.getFunction().equals("Hump")) 
       fitnessValue = hump(present);    
   return fitnessValue;
 }

 
 
 /**
   * Update the current position of each agent in the popoluation
   * using the standard constriction PSO)
   */

protected void update() {

   // do the normal constriction PSO stuff with species seeds as lbests
   for (int k=0; k < model.getDimension(); k++) 
   {
     
         //if (model.getMethod().equals("DCPSO")) 
         {              
            v[k] = 0.729843788*(v[k]
                + 2.05 * model.getRandomUniform() * (pBest[k] - present[k]) 
                + 2.05 * model.getRandomUniform() * (nBest[k] - present[k]));
         }
         
         // Adjust the velocity vector so that the agent cannot go
         // beyond the search space, preventing the population from explosion
         if (v[k] > model.getVelocityMax()) 
              v[k] = model.getVelocityMax();
         else if (v[k] < -model.getVelocityMax()) 
              v[k] = - model.getVelocityMax();
         
         // Based on the new velocity, update the agent's position
         present[k] = present[k] + v[k];
         
         // Obtain right range and left range from PSOParameters object
         double rightRange = model.getRightRange();
         double leftRange = model.getLeftRange();
            
         // Adjust the position vector so that the agent's new location varies
         // It is hoped the method below increases the overall population diversity

         // present[k] is not set to the boundary, since the CPSO and other variants should always converge
         // if c1, c2, and c3 are chosen appropriately (sum to 4.1 according to Clerc). 
         // Setting it the boundary would also reduce the influence of the differential vector.    
         
         if (model.getFunction().equals("SixHumpCamel") && (k == 1))       
         {   
            rightRange = 1.1; 
            leftRange = -1.1;
         }                 
         
         if (present[k] > rightRange) {
            //present[k] = rightRange;
            //present[k] = (present[k] + rightRange)/2.0;
            present[k] = 2*rightRange - present[k];
         }
         else if (present[k] < leftRange) {
            //present[k] = leftRange;
            //present[k] = (present[k] + leftRange)/2.0;
            present[k] = 2*leftRange - present[k];
         }
                    
   }
   
}


/**
   * Update for the fitness sharing function only (no PSO involved)
   */

protected void blxUpdate() {

    int numValues;
    double mutationRate = 0.01;
    double alpha = 0.2;   // BLX-alpha
    double max = model.getRightRange();
    double min = model.getLeftRange();
    double newMin;
    double newMax;
    double beta;
    int dice;
        
    dice = (int)(Math.floor(Math.random() * (model.agentList).size()));
    PSOAgent agent = (PSOAgent) (model.agentList).get(dice); 
        
    // crossover and mutation
    for (int k=0; k < model.getDimension(); k++) 
    {
         double rand = model.getRandomUniform();
            
         // half cross-over, half blend. 
         //if (rand < 0.5)
         //   present[k] = (agent.getPresent())[k]; // uniform crossover 50% probability
         
         //else
         {
            // BLX-alpha from Deb's Self-adaptive GA paper 2001.
            //rand = model.getRandomUniform();
            beta = (1 + 2*alpha) * rand - alpha;
            newMin = present[k] - alpha*((agent.getPresent())[k] - present[k]);
            newMax = (agent.getPresent())[k] + alpha*((agent.getPresent())[k] - present[k]);

            if ((newMin < min) && (newMax < max))
            {
                present[k] = (newMax - min) * model.getRandomUniform() + min; 
            }
            else if ((newMax > max) && (newMin > min))
            {
                present[k] = (max - newMin) * model.getRandomUniform() + newMin; 
            }
            else if ((newMax > max) && (newMin < min))
            {
                present[k] = (max - min) * model.getRandomUniform() + min; 
            }
            else
            {
                present[k] = (newMax - newMin) * model.getRandomUniform() + newMin; 
            }       
        }
         
        rand = model.getRandomUniform();
        if (rand <mutationRate)
           present[k] = (max - min) * model.getRandomUniform() + min;   
   }
}   


/**
   * Return the fitness value of this agent based on its current position
   * @return
   */
  /** Parabola **/
  /* -20 <= x_i <= 20; i = 1 */
  /* the minimum is 0.0 */
  private double parabola() {
    double total = 0;
    for (int i=0; i < model.getDimension(); i++) {
      total += (present[i] * present[i]);
    }
    return total; //minimization
  }
  


 /**
   * Return the fitness value of this agent based on its current position
   * @return
   */
  /** Sphere **/
  /* -5.12 <= x_i <= 5.12; i = 1...,30 */
  /* the minimum is 0.0 */
  private double sphere() {
    double total = 0;
    for (int i=0; i < model.getDimension(); i++) {
      total += ((present[i]) * (present[i]));
    }
    return total; //minimization
  }
  
 
  
 /**
   * Return the fitness value of this agent based on its current position
   * @return
   */
  /** Himmelblau function **/
  /* -6.0 <= x1 and x2 <= 6.0  */
 private double himmelblau(double[] present) {
 
   double y = 0;
   y = 200 - Math.pow((Math.pow(present[0], 2) + present[1] - 11), 2) 
           - Math.pow((present[0] + Math.pow(present[1], 2) - 7), 2);
   
   return y;  // maximization
 }
 
/**
 * Six-hump camel back function (F11)     
 * @return
**/
private double sixHumpCamelBack(double[] present)
{
    /* -1.9 <= x_1 <= 1.9; -1.1 <= x_2 <= 1.1 */
    double result = -((4 - 2.1*Math.pow(present[0], 2) + Math.pow(present[0], 4)/3) * Math.pow(present[0], 2)
                    + present[0]*present[1] + (-4 + 4*Math.pow(present[1], 2))*Math.pow(present[1], 2));
    return result;  // maximization
}
  
  
  
 /**
   * Shubert 2 dimensional function 
   * @return
   */
  /* -10 <= x_i <= 10 for i=1,2.  */
 private double shubert2d(double[] present) {
 
  double f, factor1, factor2;

  factor1 = 0.0;
  for(int j=1; j <= 5; j++)
     factor1 = factor1 + j * Math.cos(( j + 1)*present[0] + j);

  factor2 = 0.0;
  for(int i=1; i <= 5; i++)
    factor2 = factor2 + i * Math.cos(( i + 1)*present[1] + i);

  f = factor1 * factor2;

  return 0 - f; // maixmization
 }
 
 
  
  /**
   * Shubert 3 dimensional function 
   * @return
   */
  /* -10 <= x_i <= 10 for i=1,2,3.  */
 private double shubert3d(double[] present) {
 
  double f, factor1, factor2, factor3;

  factor1 = 0.0;
  for(int j=1; j <= 5; j++)
     factor1 = factor1 + j * Math.cos(( j + 1)*present[0] + j);

  factor2 = 0.0;
  for(int i=1; i <= 5; i++)
    factor2 = factor2 + i * Math.cos(( i + 1)*present[1] + i);

  factor3 = 0.0;
  for(int k=1; k <= 5; k++)
     factor3 = factor3 + k * Math.cos(( k + 1)*present[2] + k);
     
  f = factor1 * factor2 * factor3;

  return 0 - f; // maixmization
 }


 /**
   * Shubert 4 dimensional function 
   * @return
   */
  /* -10 <= x_i <= 10 for i=1,2,3,4.  */
 private double shubert4d(double[] present) {
 
  double f, factor1, factor2, factor3, factor4;

  factor1 = 0.0;
  for(int j=1; j <= 5; j++)
     factor1 = factor1 + j * Math.cos(( j + 1)*present[0] + j);

  factor2 = 0.0;
  for(int i=1; i <= 5; i++)
    factor2 = factor2 + i * Math.cos(( i + 1)*present[1] + i);

  factor3 = 0.0;
  for(int k=1; k <= 5; k++)
     factor3 = factor3 + k * Math.cos(( k + 1)*present[2] + k);
     
  factor4 = 0.0;
  for(int k=1; k <= 5; k++)
     factor4 = factor4 + k * Math.cos(( k + 1)*present[3] + k);
  
  f = factor1 * factor2 * factor3 * factor4;

  return 0 - f; // maixmization
 }



  
 /**
   * Foxholes function (De Jong F5)
   * @return
   */
  /* -65.536 <= x and y <= 65.535  */
 private double foxholes(double[] present) {
 
      int[][] a = new int[2][5];
      int next1 = -32;
      int next2 = -32;
      
      for (int j = 0; j < 5; j++)
      {
         a[0][j] = next1;
         a[1][j] = next2;
         
         next1 += 16;
         next2 += 16;
      }
         
      double solution = 500.0;
      double sum = 0.0;
      
      for (int j = 0; j < 25; j++)
          sum = sum + (1.0 / f5_j(present, j, a));         
       
      return (500.0 - (1.0 / (sum + .002)));
 }
 
 private double f5_j(double[] present, int j, int[][] a)
 {
    double temp = present[0] - a[0][j%5];
    double temp2 = temp * temp;
    double temp6 = temp2 * temp2 * temp2;
    double save = temp6;
    temp = present[1] - a[1][j/5];
    temp2 =  temp * temp;
    temp6 = temp2 * temp2 * temp2;
                 
    return (save + temp6 + (j + 1)); 
 }
  
 
 
  // Deb1 - 5 global optima with an equal height (evenly spaced)
  private double deb1(double[] present) {
 
  double y = 0;
  int i = 0;
  
  if ((present[i] >= 0.0) && (present[i] <= 1.0))  
    y = Math.pow(Math.sin(5*Math.PI*present[i]), 6);
  else
    y = -100.0;
  return y; 
 }

 // Deb2 - 1 global optimum and 4 local optima (evenly spaced)
 private double deb2(double[] present) {
 
   double y = 0;
   int i = 0;
   
   if ((present[i] >= 0.0) && (present[i] <= 1.0))  
      y = Math.exp(-2*Math.log(2)*Math.pow((present[i] - 0.1)/0.8, 2)) * Math.pow(Math.sin(5*Math.PI*present[i]), 6);
   else
      y = -100.0;
   return y;
 }

 // Deb3 - 5 global optima (of an equal height) unevenly spaced
 private double deb3(double[] present) {
 
   double y = 0;
   int i = 0;
   if ((present[i] >= 0.0) && (present[i] <= 1.0))  
        y = Math.pow(Math.sin(5*Math.PI*(Math.pow(present[i], 3.0/4.0) - 0.05)), 6);
   else
        y = -100.0;
   return y;
 }

 // Deb4 - 1 global optimum and 4 local optima (unevenly spaced)
private double deb4(double[] present) {
 
   double y = 0;
   int i = 0;
   if ((present[i] >= 0.0) && (present[i] <= 1.0))  
        y = Math.exp(-2*Math.log(2)*Math.pow((present[i] - 0.08)/0.854, 2)) 
            * Math.pow(Math.sin(5*Math.PI*(Math.pow(present[i], 3.0/4.0) - 0.05)), 6);
   else
        y = -100.0;
   return y;
 }
 
 
 
/**
   * Inverted Vincent function (assuming maximization)
   * @return
   */
  /* 0.25 <= x <= 10  */

private double vincent1d(double[] present) {
 
   double y = 0;
   double sum = 0;
   
   for(int i=0; i < model.getDimension(); i++)
        sum += Math.sin(10*Math.log(present[i]));
      
   y = 1.0/model.getDimension() * sum;
   return y;
 }
 
 
 /**
   * Inverted Vincent function (assuming maximization)
   * @return
   */
  /* 0.25 <= x <= 10  */

private double vincent2d(double[] present) {
 
   double y = 0;
   double sum = 0;
   
   for(int i=0; i < model.getDimension(); i++)
        sum += Math.sin(10*Math.log(present[i]));
      
   y = 1.0/model.getDimension() * sum;
   return y;
 }
 
 
  /**
   * Inverted Vincent function (assuming maximization)
   * @return
   */
  /* 0.25 <= x <= 10  */

private double vincent3d(double[] present) {
 
   double y = 0;
   double sum = 0;
   
   for(int i=0; i < model.getDimension(); i++)
        sum += Math.sin(10*Math.log(present[i]));
      
   y = 1.0/model.getDimension() * sum;
   return y;
 }
 
 
 /**
   * Two-Peak Trap
   * @return
   */
  /* 0 <= x <= 20  */
 
private double twoPeakTrap(double[] present) {
   double y = 0;
   int i = 0;
   
   if ((present[i] >= 0) && (present[i] < 15))
        y = 160.0/15.0 * (15.0 - present[i]);
   else if ((present[i] >= 15) && (present[i] <= 20))
        y = 200.0/5.0 * (present[i] - 15.0);
   else
        y = -100.0;     
   return y;
 }


/**
   * Central Two-Peak Trap
   * @return
   */
  /* 0 <= x <= 20  */
private double centralTwoPeakTrap(double[] present) {
   double y = 0;
   int i = 0;
   
   if ((present[i] >= 0) && (present[i] < 10))
        y = 160.0/10.0 * present[i];
   else if ((present[i] >= 10) && (present[i] < 15))
        y = 160.0/5.0 * (15.0 - present[i]);
   else if ((present[i] >= 15) && (present[i] <= 20))
        y = 200.0/5.0 * (present[i] - 15.0);
   else
        y = -100.0;     
   return y;
 }
  
   
/**
   * Five-Peak Uneven Trap Function
   * @return
   */
  /* 0 <= x <= 30  */
 private double fiveUnevenPeaksTrap(double[] present) {
   double y = 0;
   int i = 0;
   
   if ((present[i] >= 0) && (present[i] < 2.5))
        y = 80.0 * (2.5 - present[i]);
   else if ((present[i] >= 2.5) && (present[i] < 5.0))
        y = 64.0 * (present[i] - 2.5);
   else if ((present[i] >= 5.0) && (present[i] < 7.5))
        y = 64.0 * (7.5 - present[i]);
   else if ((present[i] >= 7.5) && (present[i] < 12.5))
        y = 28.0 * (present[i] - 7.5);
   else if ((present[i] >= 12.5) && (present[i] < 17.5))
        y = 28.0 * (17.5 - present[i]);
   else if ((present[i] >= 17.5) && (present[i] < 22.5))
        y = 32.0 * (present[i] - 17.5);
   else if ((present[i] >= 22.5) && (present[i] < 27.5))
        y = 32.0 * (27.5 - present[i]);
   else if ((present[i] >= 27.5) && (present[i] < 30))
        y = 80.0 * (present[i] - 27.5);
   else
        y = -100.0;     
   return y;
 } 
 
 
  /** Rastrigin **/
  /* -5.12 <= x_i <= 5.12; i = 1...,30 */
  /* the minimum is 0.0 */
  private double rastrigin(double[] present) {
   double total = 0;
   for (int i=0; i < model.getDimension(); i++) {
     //total += (present[i] * present[i]);
     total += (present[i] * present[i])
        - 10 * Math.cos(2 * Math.PI * (present[i]))
        + 10;
   }
   return -total; // maximization
  }
  
  /** Generic Hump function **/  
 private double hump(double[] present) {
 
   // Now evaluate 'present' via the hump function   
   Iterator peakIter = model.peakList.iterator();
   double fitness =0, maxFit=0;
   boolean INSIDE = false;
   
   while (peakIter.hasNext())
   {
        Peak pk = (Peak) peakIter.next();
        double dist; 
        double sum = 0;
                       
        for(int j=0; j < model.getDimension(); j++)
            sum += Math.pow(present[j] - pk.location[j], 2);
        dist = Math.sqrt(sum);
        double h = 1.0, alpha=1.0;
        
        if (dist <= pk.r)
        {
            fitness = h*(1 - Math.pow(dist/pk.r, alpha));
            INSIDE = true;
            if (fitness > maxFit)
                maxFit = fitness;
        }
   }     
   if (!INSIDE)
        maxFit = 0.0;
   
   //System.out.println("maxFit :\n"+maxFit);
   return maxFit;
 }  
    
    
    
   
  /**
   * Drawable implementationdraw() is called whenever the display is updated,
   * assuming the agent is part of what is being displayed.
   */
  public void draw(SimGraphics g) {
    g.drawFastRoundRect(java.awt.Color.red);
  }
  
}
