
/**
 * <p>Title: PSOModel.java </p>
 * <p>Description: The population model for the rpso algorithm </p>
 * <p>Copyright: Copyright (c) 28/05/2009</p>
 * <p>Institute: School of Computer Science and IT, RMIT University</p>
 * @author Xiaodong Li (xiaodong.li@rmit.edu.au)
 * @version 1.0
 * This is the implementation of the rpso algorithms described in the following paper:
 * Li, X. (2010), "Niching without Niching Parameters: Particle Swarm Optimization using a Ring Topology", 
 * IEEE Transactions on Evolutionary Computation,  Volume 14, No.1, February 2010, pp.150-169.
 * For more information, please refer to: http://goanna.cs.rmit.edu.au/~xiaodong/
 * You need to have RePast 3.1 installed on your machine in order to compile and run the program. 
 * For more informaton on RePast, see: http://repast.sourceforge.net/repast_3/index.html
 */
 
import java.util.Vector;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.ListIterator;
import java.awt.Color;
import java.util.Collections;
import java.io.*;
import java.math.*;
import java.lang.System;

import cern.jet.random.*;
import uchicago.src.sim.util.Random;
import uchicago.src.sim.util.*;
import uchicago.src.sim.space.*;
import uchicago.src.sim.engine.*;
import uchicago.src.sim.gui.*;
import uchicago.src.sim.analysis.*;
import uchicago.src.reflector.*;
import java.util.Date;


// Simulation models should extend SimModelImpl which does some basic setup
// and allows a controller easy access to a simulations initial parameters.
public class PSOModel extends SimModelImpl {

  // -------------------------------------------------------------- Members

  /** A schedule associated with this model **/
  protected Schedule schedule;
  
  /** A list of all existing agents, and peaks for the hump func **/
  protected ArrayList agentList, peakList;;
  
  /** A DataRecorder allows data generated by a simulation to be written to a file **/
  protected DataRecorder recorder;

  /** A plot to display particles in the decision space, and niche plot to show clusters  **/
  protected Plot aPlot, nichePlot;
  
  /** A graph to display the fitness at each iteration**/
  protected OpenSequenceGraph graph; 
  
  /** The array stores the location of the neighbourhood best **/
  private double[] nBest;

  /** The value of the local best **/
  private double nBestValue = -1.7976931348623157E308;

  /** A list of all existing nBest **/
  protected ArrayList nBestList;
      
  /** Number of evaluations **/
  protected int numEvals = 0;
  
  /** Accuracy threshold **/
  private double threshold = 0.0001; //0.2;
  
  /** Number of run **/
  protected int run = 50;

  /* parameters accessor methods */
  private int numParticles = 50; 
  
  /* number of peaks for the Hump func */
  private int numPeaks = 10;
  
  /* please click the "reset" button to choose different settings for ranges and v_max */
  private double leftRange = 0; //-5.12; //-10; //0.25; //-10; // -65.536; //-1.9; //-6; //0.25; //0;  //-20;  //-5.12; 
  private double rightRange = 1.0; //5.12; //10; //10; //65.535; //1.9; //6; //10; //1;  //20; //5.12;
  private double velocityMax = 0.5; // 5.12; //10; //4.875; //63.535; //4.875; //3; //10;  //5.12;
  private int maxIterations = 10000;
  private int maxNumEvals = 200000;
  private int dimension = 1;  
  private String function = "Deb5Even"; //"Hump"; //"TwoPeakTrap"; //"Deb5Even"; //"Rastrigin"; //"Shubert2D"; //"Vincent2D";  //"CentralTwoPeakTrap"; //"Shubert3D";  //"Foxholes"; // "SixHumpCamelBack"; //"Himmelblau"; //"Vincent1D"; //"Deb5Even"; //"FiveUnevenPeaksTrap";//"FiveUnevenPeaksTrap";  
  public String method = "r3PSO"; 
  private double radius = 0.5; //0.29;  /** species radius for performance measurement only **/
  private int currentpos = 0;

  static private boolean BATCH = false;  // Batch or graph mode
  private boolean SHUFFLE = false; // shuffle the agentList or not
  private int allPeaksFound = 0; // flag if all peaks are found (ie., a success)
  private int numPeaksFound = 0; // exactly the number of peaks found at the end of a run
  
  private Date startTime; // start time for a run
  private Date endTime;  // end time for the run
  private double elapsedTime=0; // time taken
  
  // ---------------------------------------------------------- Constructor(s)

  public PSOModel() {
    
    Vector v = new Vector();
    v.add("r2PSO"); 
    v.add("r2PSO_lhc");
    v.add("r3PSO");
    v.add("r3PSO_lhc");
    v.add("gBestPSO");
    v.add("SPSO");
    v.add("FitShare");

    ListPropertyDescriptor pd = new ListPropertyDescriptor("Method", v);
    descriptors.put("Method", pd);
        
    Vector v2 = new Vector();
    //v2.add("Parabola");
    //v2.add("Sphere");
    v2.add("Himmelblau");
    v2.add("SixHumpCamelBack");
    v2.add("Shubert2D");
    v2.add("Shubert3D");
    v2.add("Shubert4D");
    v2.add("Foxholes");
    v2.add("Deb5Even");
    v2.add("Deb5Uneven");
    v2.add("Deb1Even");
    v2.add("Deb1Uneven");
    v2.add("TwoPeakTrap");
    v2.add("CentralTwoPeakTrap");
    v2.add("FiveUnevenPeaksTrap");
    v2.add("Rastrigin");
    v2.add("Vincent1D");
    v2.add("Vincent2D");
    v2.add("Vincent3D");
    v2.add("Hump");
    ListPropertyDescriptor pd2 = new ListPropertyDescriptor("Function", v2);
    descriptors.put("Function", pd2);
  }


  // ------------------------------------------------------------- Methods

  /** Every model must have begin() and setup() methods (required for implementing
    * the SimModel inteface).
    * begin() should intialize the model for the start of a run. Consequently,
    * the build* methods are called here, and any displays are displayed. build()
    * is called whenever the start button (or the step button if the run has
    * not yet started) is clicked.
    */
  public void begin() {
  
    // initialize Random.uniform
    Random.createUniform();
  
    Random.createNormal(0,1.0);
    
    nBestValue = -1.7976931348623157E308;
    
    startTime = new Date();
    //System.out.println("  Start Time : " + startTime);
    
    // build the model
    buildModel();
    
    // build the schedule
    buildSchedule();

    if (BATCH == false)
    {
    // build the display
    buildDisplay();
 
    // Display the plot
    aPlot.display();
    
    // Display the plot
    nichePlot.display();
    
    // Display the graph
    graph.display();
    }
  }

  /**
   * setup() prepares the model for another run. Called whenver the setup button
   * is clicked. Setup should set any objects that are created over the course
   * of the run to null, and dispose of any DisplaySurfaces or graphs. While
   * not strictly necessary this should some prevent memory leaks and calling
   * System.gc() helps too. The initial parameters should be set to whatever
   * defaults the user wants to see initially.
   */
  public void setup() {
    
    // Initialize all the variables    
    schedule = null;
    
    // set the function ranges, velocityMax etc to the chosen one
    setFunctionParams();
        
    // Create an array list contains all agents;
    agentList = new ArrayList();
    
    // hump function setting up
    if (getFunction().equals("Hump"))
    {
        peakList = new ArrayList(); 
        humpFunc(); // setting up hump
    }             
    // Initialize the nBestList
    nBestList = new ArrayList();

    // Initialize the nBest[] and nBest value
    nBest = new double[dimension];
    nBestValue = -1.7976931348623157E308;
    numEvals = 0; // counting starts from 0
       
    if (aPlot != null)
      aPlot.dispose();
    aPlot = null;
    
    if (nichePlot != null)
      nichePlot.dispose();
    nichePlot = null;
  
    if (graph != null) {
      graph.dispose();
      graph = null;
    } 

    // Create the plot for the decision space
    aPlot = new Plot("Decision space plot");
    for (int i=0; i < agentList.size(); i++)
        aPlot.addLegend(i, "current", Color.BLACK, aPlot.CIRCLE);
    aPlot.addLegend(1001, "pBest", Color.RED, aPlot.CIRCLE);
    aPlot.addLegend(1003, "nBest", Color.BLUE, aPlot.CROSS);
    
    // Create a niche plot to show clusterings
    nichePlot = new Plot("Index-Position plot");
    nichePlot.addLegend(1001, "pBest", Color.RED, nichePlot.CIRCLE);
    nichePlot.addLegend(1003, "nBest", Color.BLUE, nichePlot.CROSS);
    
    // new sequence graph
    graph = new OpenSequenceGraph("nBest Graph", this);
    this.registerMediaProducer("nBest Graph", graph);
     
    // create a schedule with an interval of one.
    schedule = new Schedule(); 
    
    System.gc();
     
  }

  /**
   * A required method, returns the schedule associated with this model
   */
  public Schedule getSchedule() {
    return schedule;
  }

  /**
   * a required method - displayed on the Controller toolbar.
   */
  public String getName() {
    return "PSO";
  }


  /**
   * The typical way to code a simulation is to divide up the creation of the
   * simulation into three methods - buildModel(), buildDisplay(),
   * buildSchedule(). This division is not strictly necessary, but it does
   * make the creation conceptually clearer.
   *
   * The buildModel() method is responsible for creating those parts of the
   * simulation that represent what is being modeled. Consequently, the agents
   * and their environment are typically created here together with any
   * optional data collection objects. Of course, this method may call other
   * methods to help build the model.
   */
  public void buildModel() {
    System.out.println("Build Model");     
    
    // We start off the simulation by adding new agent into the model
    for (int i = 0; i < numParticles; i++) 
    {   
        PSOAgent agent = new PSOAgent(this);
        // Add this agent into the agent list
        agentList.add(agent);       
    }
    
    if (BATCH ==  true)
    {
        // create a new DataRecorder and write the data to ./sugar_data.txt
        recorder = new DataRecorder("./data.txt", this);     
        
        // Adds a data source that records nBestValue.
        //recorder.addNumericDataSource("nBestValue", new nBestValueSource());
        //recorder.addNumericDataSource("numEvals", new numEvalsSource());
        
        //recorder.addNumericDataSource("allPeaksFound", new allPeaksFoundSource());
        recorder.addNumericDataSource("elapsedTime", new elapsedTimeSource());
        recorder.addNumericDataSource("numEvals", new numEvalsSource());
        recorder.addNumericDataSource("numPeaksFound", new numPeaksFoundSource());
    }
  }

  /**
   * data source for recording nBestValue
   */
  class nBestValueSource implements NumericDataSource {
    public double execute() {
      return nBestValue;      
    }
  }

 /**
   * data source for recording elapsedTime
   */
  class elapsedTimeSource implements NumericDataSource {
    public double execute() {
      return elapsedTime;      
    }
  }

 /**
   * data source for recording if allPeaksFound
   */
  class allPeaksFoundSource implements NumericDataSource {
    public double execute() {
      return allPeaksFound;      
    }
  }

  /**
   * data source for recording numEvals
   */
  class numEvalsSource implements NumericDataSource {
    public double execute() {
      return numEvals;      
    }
  }
 
   /**
   * data source for recording numPeaksFound
   */
  class numPeaksFoundSource implements NumericDataSource {
    public double execute() {
      return numPeaksFound;      
    }
  }


 
  /**
   * buildDisplay() builds those parts of the simulation that have to do with
   * displaying the simulation to a user.
   */
  protected void buildDisplay() {

    // Create a display to display the agentGrid on the screen.
    // This will display any Drawable contained within the Object2DGrid
    // In a non-batch simulation the agents in a simulation will typically
    // implement the Drawable interface.

    graph.setXRange(0, 200);
    graph.setYRange(0, 100);
    graph.setAxisTitles("Iteration", "nBest Value");
    
    // have the graph create the sequence for us from the getNumTriggered method.
    
    graph.addSequence("nBest Value", new Sequence() {
        public double getSValue() {
         return nBestValue;
    }
    });       
  }



  /**
   * buildSchedule builds the schedule that changes the simulations state.
   * Under this scheme, a simulation is a state machine where all transitions
   * between states are the result of actions initiated by a schedule.
   */
  public void buildSchedule() {
     // For each iteration (tick) of the simulation, call the step() method
     // of this PSOModel object to update the update the position,
     // pbest, nBestt of the particle swarm
     
     schedule.scheduleActionBeginning(0, this, "step");
     if (BATCH == true)
     {
        schedule.scheduleActionAtEnd(recorder, "record");
        schedule.scheduleActionAtEnd(recorder, "writeToFile");
     }
     schedule.scheduleActionAtEnd(this, "writeStat");
  }

 
  public void writeStat() {
    //System.out.println("offline_error : ");
    endTime = new Date();
    //System.out.println("  End Time  : " + endTime);
    elapsedTime = endTime.getTime() - startTime.getTime();
    System.out.println("That took " + elapsedTime + " milliseconds");
    
  }
 
  /**
   * This step() method implements the PSO algorithm as stated below
   *
   * (a):  v[] = ki * (v[] + c1 * rand() * (pBest[] - present[])
   *               + c2 * rand() * (nBest[] - present[]))
   * (b):   present[] = persent[] + v[]
   *
   *    Do
   *      For each particle
   *      Calculate fitness value
   *        If the fitness value is better than the best fitness value (pBest) in history
   *        set current value as the new pBest
   *   End
   *
   *   Choose the particle with the best fitness value of all the particles as the nBestt
   *
   *   For each particle
   *        Calculate particle velocity according equation (a)
   *        Update particle position according equation (b)
   *   End
   *  While maximum iterations or minimum error criteria is not attained
   *
   */
  public void step() {

    // If hit the maximum genereration then stop
    //if (this.getTickCount() > maxIterations) 
    if (numEvals > maxNumEvals)
    {
        System.out.println("numEvals : "+numEvals);
        this.stop();
    }

    // smiple 2 particles neighbourhood, of a ring topology
    if (getMethod().equals("r2PSO") || getMethod().equals("r2PSO_lhc")) 
        ring2();
    
    // smiple 2 particles neighbourhood, of a ring topology, using deterministic crowding
    if (getMethod().equals("r2ssdcPSO") || getMethod().equals("r2ssdcPSO_lhc")) 
        ring2dc();      
    
    // a typical ring topology lbest PSO (with 3 members)
    if (getMethod().equals("r3PSO") || getMethod().equals("r3PSO_lhc") )
        ring3();
            
    // a standard gBest PSO 
    if (getMethod().equals("gBestPSO"))
        gBest();
    
    // a standard SPSO (using pBest as seed for each species) 
    if (getMethod().equals("SPSO"))
        spso();
    
    // fitness sharing method
    if (getMethod().equals("FitShare"))
        fs();
     
    // get a list of pBests sorted from the best-fit to the least-fit    
    ArrayList pBestList = new ArrayList();
    pBestList = getPBestList(agentList);
    //System.out.println("pBestList size : "+pBestList.size());
   
    // get a list of species seeds used for performance measurement
    ArrayList seedList = new ArrayList();
    seedList = getSeeds(pBestList);   
     
    numPeaksFound = 0;
    allPeaksFound = 0;
    double[] seed = new double[dimension];
    if (numEvals > maxNumEvals)
    {
        for(int i=0; i<seedList.size(); i++)
        {
            PSOAgent p = (PSOAgent)seedList.get(i);
            if (((getFunction().equals("Shubert4D")) && ((39303.55 - p.getSeedFit()) <= getThreshold()))
             || ((getFunction().equals("Vincent3D")) && ((1.0 - p.getSeedFit()) <= getThreshold()))
             || ((getFunction().equals("Rastrigin")) && ((0.0 - p.getSeedFit()) <= getThreshold())))
                numPeaksFound++;
            //seed = p.getSeed();
            //System.out.println("seed "+i+","+p.getSeedFit()+","+seed[0]+","+seed[1]);
        }
        if (getFunction().equals("Hump"))
            checkHump();
        System.out.println("numPeaksFound: "+numPeaksFound);
    }
    
    // check to see if expected acuracy is reached, and if so, stop the run
    checkAcuracy(seedList);       
         
    if (BATCH == false)
    {           
        
        // Clear the plot first 
        for (int i=0; i < agentList.size(); i++)
            aPlot.clear(i);
        aPlot.clear(1001);  
        aPlot.clear(1003);
        nichePlot.clear(1001);
        nichePlot.clear(1003);
          
        // Display a plot to show all particles in the decision space
        aPlot.setConnected(true);
        for (int i = 0; i < agentList.size(); i++) {   //step through each particle
            PSOAgent p = (PSOAgent)agentList.get(i);
            p.positionPlot(i);
        }
        
        // Display all pBest positions
        aPlot.setConnected(false);
         
        // Display all pBest and nBest positions        
        for (int i=0; i < agentList.size(); i++) 
        {  
            PSOAgent p = (PSOAgent)agentList.get(i);
            p.pBestNBestPlot();
        }
        
        // Display all particles (in index-and-position space) 
        nichePlot.setConnected(false);
        for (int i = 0; i < agentList.size(); i++) {   //step through each particle
            PSOAgent p = (PSOAgent)agentList.get(i);
            p.indexAndPosition(i);
        }        
        
        aPlot.fillPlot();
        nichePlot.fillPlot();
        aPlot.updateGraph();  
        nichePlot.updateGraph();
        
        // draw the fitness plot
        graph.step();     
    
    }// if batch==false
    //System.out.println("agentList size: "+agentList.size());  
    
  }
 
/**
  * A simple gBest PSO
  * 
  */
  public void gBest()
  {
    // shuffle the agentList first 
    int agentListSize = agentList.size();
    
    // Iterate through the list of agents and evaluate the fitness        
    for (int i = 0; i < agentListSize; i++) 
    {
        PSOAgent agent = (PSOAgent) agentList.get(i);
        
        // Calculate the agent fitness value as well as setting pBest 
        agent.step(); 
        numEvals++;    
    }
    
    // Iterate through the list of agents and decide the nBest        
    for (int i = 0; i < agentListSize; i++) 
    {
        PSOAgent agent = (PSOAgent) agentList.get(i);
    
        // Obtain its pBest fitness value 
        double pBestValue = agent.getPBestValue();
            
        // Choosing the leader for agent 
        if (pBestValue > nBestValue)
        {
            // Update the nBest value
            nBestValue = pBestValue;            
            // Update the nBest location
            setNBest(agent.getPBest());   
        }
        
    }
       
    // update gBest for all particles     
    for (int i = 0; i < agentListSize; i++) 
    {
        PSOAgent agent = (PSOAgent) agentList.get(i);
        agent.setNBestValue(nBestValue);
        agent.setNBest(nBest);
        agent.update();
    }
    
  }

/**
  * Speciation-based PSO (SPSO)
  * 
  */
  public void spso()
  {
    int agentListSize = agentList.size();
    
    // Iterate through the list of agents and evaluate the fitness        
    for (int i = 0; i < agentListSize; i++) 
    {
        PSOAgent agent = (PSOAgent) agentList.get(i);
        
        // Calculate the agent fitness value as well as setting pBest 
        agent.step(); 
        numEvals++;    
    }
    
    // get a list of pBest sorted in the order from the best-fit to the least-fit
    ArrayList pBestList = new ArrayList();
    pBestList = getPBestList(agentList);
    
    // get a list of species seeds used for performance measurement
    ArrayList seedList = new ArrayList();
    seedList = getSeeds(pBestList);  

    // Note that SPSO uses seeds for optimization (unlike other variants)   
    // update with nBest values for all particles     
    for (int i = 0; i < agentListSize; i++) 
    {
        PSOAgent agent = (PSOAgent) agentList.get(i);
        agent.setNBestValue(agent.getSeedFit());
        agent.setNBest(agent.getSeed());
        agent.update();
    }
    
  }

/**
  * fitness sharing 
  * 
  */
  public void fs()
  {  
    double shFunc;
    double shFit;
    double euDist;
    double nc;
    
    int agentListSize = agentList.size();
    
    // Iterate through the list of agents and evaluate the fitness        
    for (int i = 0; i < agentListSize; i++) 
    {
        PSOAgent agent = (PSOAgent) agentList.get(i);  
        agent.step(); // calculate fitness value 
        numEvals++;
        nc = 0.0;   
        for (int j = 0; j < agentListSize; j++) 
        {
            PSOAgent agent2 = (PSOAgent) agentList.get(j);  
            euDist = getEuDist(agent.getPresent(), agent2.getPresent());
            //System.out.println("euDist: "+j+": "+euDist);
            if (euDist <= radius)
                shFunc = 1.0 - Math.pow(euDist/radius, 2);
            else
                shFunc = 0;
            nc += shFunc;
        }
        shFit = agent.getFitnessValue()/nc;
/*        System.out.println("present[0]: "+i+": "+(agent.getPresent())[0]);
        System.out.println("fitness: "+i+": "+agent.getFitnessValue());
        System.out.println("shFit: "+i+": "+shFit);
        System.out.println("nc: "+i+": "+nc);
*/
        agent.setShFit(shFit);    
    }

    double sumfitness=0.0;
    for(int i = 0; i < agentListSize; i++) 
    {
        PSOAgent agent = (PSOAgent) agentList.get(i);
        sumfitness += agent.getShFit();       
    }
    
    // proportionate selection
    ArrayList newAgentList = new ArrayList();    
    for(int i = 0; i < agentListSize; i++) 
    {      
        PSOAgent agent = (PSOAgent) agentList.get(select(sumfitness));    
        // Add this agent into the new agent list
        //System.out.println("selected individual: "+select(sumfitness));
        newAgentList.add(agent);       
    }
    
    agentList.clear();
    agentList.addAll(newAgentList);    
    
    // update according to real-coded mutation and crossover (including BLX-alpha)    
    for(int i = 0; i < agentListSize; i++) 
    {    
        PSOAgent agent = (PSOAgent) agentList.get(i);    
        //System.out.println("shFit: "+agent.getShFit());   
        agent.blxUpdate();    
    }
    
  }
 

/* roulette-wheel selection */
public int select(double sumfitness)
{  
    // roulette-wheel selection
    double sum, pick;
    pick = getRandomUniform();
    sum = 0;
    int i;

    if(sumfitness != 0)
    {
        for(i = 0; (sum < pick) && (i < agentList.size()); i++)
        {
            PSOAgent agent = (PSOAgent) agentList.get(i);
            sum += agent.getShFit()/sumfitness;
        }
    }
    else
        i = (int)(Math.floor(Math.random() * agentList.size()))+1;
    
    return (i-1);
}  
  
  
  
 
/**
  * ring2() shows a simple ring topology based lbest PSO
  * the i-th particle interacts only with the (i+1)th particle
  */
  public void ring2()
  {
    // shuffle the agentList first 
    if (SHUFFLE == true)
        Collections.shuffle(agentList);
    int agentListSize = agentList.size();
    
    // Iterate through the list of agents and evaluate the fitness        
    for (int i = 0; i < agentListSize; i++) 
    {
        PSOAgent agent = (PSOAgent) agentList.get(i);
        
        // Calculate the agent fitness value as well as setting pBest 
        agent.step(); 
        numEvals++;    
    }
    
    // Iterate through the list of agents and decide the nBest        
    for (int i = 0; i < agentListSize; i++) 
    {
        PSOAgent agent = (PSOAgent) agentList.get(i);
    
        // Obtain its pBest fitness value 
        double pBestValue = agent.getPBestValue();
        
        // Update the nBest value
        nBestValue = pBestValue;            
        // Update the nBest location
        setNBest(agent.getPBest());
        
        // Pick another particle to interact with above agent. We simply use the 
        // index of the individuals so that a local leader can be chosen to climb
        // a hill locally. However, since there is an overlap in index, selection 
        // pressure (towards local peaks) will propagate very quickly through the population.
        PSOAgent agent2 = new PSOAgent(this);
        if (i < (agentListSize - 1))
            agent2 = (PSOAgent) agentList.get(i+1);
        else if (i >= (agentListSize - 1))
            agent2 = (PSOAgent) agentList.get(0); // connect back to the first one
        
        if (getMethod().equals("r2PSO_lhc")) 
            i++; 
        
        double pBestValue2 = agent2.getPBestValue();
        
        // Choosing the local leader for agent 1 and 2
        if (pBestValue2 > nBestValue)
        {
            // Update the nBest value
            nBestValue = pBestValue2;            
            // Update the nBest location
            setNBest(agent2.getPBest());   
        }
        
        // both particles share the same local best
        agent.setNBest(nBest);
        agent.setNBestValue(nBestValue);
        //System.out.println("nBestValue : "+nBestValue);
        //System.out.println("nBest: ["+nBest[0]+","+nBest[1]+"]");
        
        if (getMethod().equals("r2PSO_lhc")) 
        {
            agent2.setNBest(nBest);
            agent2.setNBestValue(nBestValue);
        }
        
        // immediately update particles (steady-state alike), instead of later in another for loop.
        //if (getMethod().equals("r2ssPSO") || getMethod().equals("r2PSO_lhc"))
        //    agent.update();
    }
       
    // update gBest for all particles     
    for (int i = 0; i < agentListSize; i++) 
    {
        PSOAgent agent = (PSOAgent) agentList.get(i);
        agent.update();
    }
    
  }
  
  /**
  * ring2dc() shows a simple ring topology based lbest PSO, 
  * using deterministic crowding 
  */
  public void ring2dc()
  {
    // shuffle the agentList first 
    if (SHUFFLE ==  true)
        Collections.shuffle(agentList);
    int agentListSize = agentList.size();
    
    // Iterate through the list of agents and do the following        
    for (int i = 0; i < agentListSize; i++) 
    {
        PSOAgent agent = (PSOAgent) agentList.get(i);
        
        // Calculate the agent fitness value as well as setting pBest 
        if (this.getTickCount() == 1)
        {
            agent.step(); // only calculate the fitness based on present[]
            numEvals++;    
        }
        // Obtain its pBest fitness value 
        double pBestValue = agent.getPBestValue();
        
        // Pick another particle to interact with above agent. We simply use the 
        // index of the individuals so that a local leader can be chosen to climb
        // a hill locally. However, since there is an overlap in index, selection 
        // pressure (towards local peaks) will propagate very quickly through the population.
        PSOAgent agent2 = new PSOAgent(this);
            
        if (i < (agentListSize - 1))
            agent2 = (PSOAgent) agentList.get(i+1);
        else if (getMethod().equals("r2ssdcPSO") && i >= (agentListSize - 1))
            agent2 = (PSOAgent) agentList.get(0); // connect back to the first one
        else if (getMethod().equals("r2ssdcPSO_lhc")) 
            i++; 
                
        // Calculate the fitness first   
        if (this.getTickCount() == 1)
        {
            agent2.step();
            numEvals++;    
        }
        double pBestValue2 = agent2.getPBestValue();
        
        // Choosing the local leader for agent 1 and 2
        if (pBestValue > pBestValue2)
        {
            // Update the nBest value
            nBestValue = pBestValue;
            
            // Update the nBest location
            setNBest(agent.getPBest());   
        }
        else  // otherwise set nBest to agent2
        {
            nBestValue = pBestValue2;
            setNBest(agent2.getPBest());   
        }
        
        // both particles share the same local best
        agent.setNBest(nBest);
        agent.setNBestValue(nBestValue);
        agent2.setNBest(nBest);
        agent2.setNBestValue(nBestValue);
        //System.out.println("nBestValue : "+nBestValue);
        //System.out.println("nBest: ["+nBest[0]+","+nBest[1]+"]");
        
        agent.update();
        agent2.update();
    
        // The following codes reset the pBest of both agent and agent2, 
        // based on the latest updates (ie., pso equations called here).
        // Below A denotes a particle's pBest position, and C denotes the 
        // child produced from A (via calling update()). Among the two particles
        // A1 and A2, If a child (either C1 or C2) is closer and fitter than 
        // the parent, then we reset the parent A's pBest position to this child's.
        // This procedure is similar to the Deterministic Crowding by Mahfoud S. W. (1995).
      
        double distA1C1, distA2C2, distA1C2, distA2C1;
               
        distA1C1 = getEuDist(agent.getPBest(), agent.getPresent());
        distA2C2 = getEuDist(agent2.getPBest(), agent2.getPresent());       
        
        distA1C2 = getEuDist(agent.getPBest(), agent2.getPresent());
        distA2C1 = getEuDist(agent2.getPBest(), agent.getPresent());
       
        agent.step();
        numEvals++; // called computeFitness once  
        agent2.step();
        numEvals++; // called computeFitness again 
        
        if ((distA1C1 + distA2C2) <= (distA2C1 + distA1C2))
        {
            if (agent.getFitnessValue() > agent.getPBestValue())
            {
                agent.setPBest(agent.getPresent());
                agent.setPBestValue(agent.getFitnessValue());
            }            
            if (agent2.getFitnessValue() > agent2.getPBestValue())
            {
                agent2.setPBest(agent2.getPresent());
                agent2.setPBestValue(agent2.getFitnessValue());
            }      
        }
        else
        {
            if (agent2.getFitnessValue() > agent.getPBestValue())
            {
                agent.setPBest(agent2.getPresent());
                agent.setPBestValue(agent2.getFitnessValue());
            }    
            if (agent.getFitnessValue() > agent2.getPBestValue())
            {
                agent2.setPBest(agent.getPresent());
                agent2.setPBestValue(agent.getFitnessValue());
            }         
        }      
    }
  }
    
    
 /**
  * ring3() shows a typical ring topology based lbest PSO, 
  * each particle interacts with the 2 particle on each side
  */
  public void ring3()
  {
  
    // shuffle the agentList first 
    if (SHUFFLE ==  true)
        Collections.shuffle(agentList);
    int agentListSize = agentList.size();
    
    // Iterate through the list of agents to evaluate fitness        
    for (int i = 0; i < agentListSize; i++) 
    {
        PSOAgent agent = (PSOAgent) agentList.get(i);
        
        // Calculate the fitness first   
        agent.step(); // only calculate the fitness based on present[]
        numEvals++;
    }
    
    // Iterate through the list of agents to decide nBest
    for (int i = 0; i < agentListSize; i++) 
    {
        PSOAgent agent = (PSOAgent) agentList.get(i);
    
        // Obtain its pBest fitness value 
        double pBestValue = agent.getPBestValue();
        
        // Update the nBest value
        nBestValue = pBestValue;            
        // Update the nBest location
        setNBest(agent.getPBest());
                 
        PSOAgent agent2 = new PSOAgent(this);
        if (i < (agentListSize - 1))
            agent2 = (PSOAgent) agentList.get(i+1);
        else if (i >= (agentListSize - 1))
            agent2 = (PSOAgent) agentList.get(0); // connect back to the first one
 
        PSOAgent agent3 = new PSOAgent(this);
        if (i < 1)
            agent3 = (PSOAgent) agentList.get(agentListSize - 1);
        else if (i <= (agentListSize - 1))
            agent3 = (PSOAgent) agentList.get(i-1);
        
        //i = i + 1; // only one particle overlapped
        // 3 particles in one isolated group for the local hill climber variant, no overlap. 
        if (getMethod().equals("r3PSO_lhc")) 
            i = i + 2; 
        
        double pBestValue2 = agent2.getPBestValue();
        
        // Choosing the local leader for agent 1, 2 and 3
        if (pBestValue2 > nBestValue)
        {
            // Update the nBest value
            nBestValue = pBestValue2;
            // Update the nBest location
            setNBest(agent2.getPBest());   
        }
   
        double pBestValue3 = agent3.getPBestValue();
        
        // Choosing the local leader for agent 1, 2 and 3
        if (pBestValue3 > nBestValue)
        {
            // Update the nBest value
            nBestValue = pBestValue3;            
            // Update the nBest location
            setNBest(agent3.getPBest());   
        }
           
        // set the local leader for the particle and its immediate neighbours
        agent.setNBest(nBest);
        agent.setNBestValue(nBestValue);
       
        if (getMethod().equals("r3PSO_lhc")) 
        {
            agent2.setNBest(nBest);
            agent2.setNBestValue(nBestValue);
            agent3.setNBest(nBest);
            agent3.setNBestValue(nBestValue);
        }
       
       // System.out.println("agent["+i+"]: "+agent.getPBestValue());
       // System.out.println("agent2: "+agent2.getPBestValue());
       // System.out.println("agent3: "+agent3.getPBestValue());
    }   
    
    // update gBest for all particles
    for (int i = 0; i < agentListSize; i++) 
    {
        PSOAgent agent = (PSOAgent) agentList.get(i);
        agent.update();
    }
  
  }
  
   
  public void setFunctionParams()
  {
    if (getFunction().equals("Parabola"))
    {
        leftRange = -20;
        rightRange = 20;
        velocityMax = 20;
        dimension = 1;
    }
    else if (getFunction().equals("Sphere"))
    {
        leftRange = -5.12;
        rightRange = 5.12;
        velocityMax = 5.12; //2.56;
        dimension = 2;
        //dimension = 100;
    }
    else if (getFunction().equals("Rastrigin"))
    {
        leftRange = -5.12;
        rightRange = 5.12;
        velocityMax = 5.12; 
        dimension = 2;
        //dimension = 100;
    } 
    else if (getFunction().equals("SixHumpCamelBack"))
    {
        leftRange = -1.9;
        rightRange = 1.9;
        velocityMax = 1.9;
        dimension = 2;
    }
    else if (getFunction().equals("Himmelblau"))
    {
        leftRange = -6.0;
        rightRange = 6.0;
        velocityMax = 6.0;
        dimension = 2;
    }
    else if (getFunction().equals("Shubert2D"))
    {
        leftRange = -10.0;
        rightRange = 10.0;
        velocityMax = 10.0;
        dimension = 2;
    } 
    else if (getFunction().equals("Shubert3D"))
    {
        leftRange = -10.0;
        rightRange = 10.0;
        velocityMax = 10.0;
        dimension = 3;
    } 
    else if (getFunction().equals("Shubert4D"))
    {
        leftRange = -10.0;
        rightRange = 10.0;
        velocityMax = 10.0;
        dimension = 4;
    } 
    else if (getFunction().equals("Foxholes"))
    {
        leftRange = -65.536;
        rightRange = 65.535;
        velocityMax = 65.536; //32.768;
        dimension = 2;
    } 
    else if (getFunction().equals("Deb5Even") || getFunction().equals("Deb5Uneven")
            || getFunction().equals("Deb1Even") || getFunction().equals("Deb1Uneven"))
    {
        leftRange = 0;
        rightRange = 1;
        velocityMax = 0.5;
        dimension = 1;
    } 
    else if (getFunction().equals("TwoPeakTrap") || getFunction().equals("CentralTwoPeakTrap"))
    {
        leftRange = 0;
        rightRange = 20;
        velocityMax = 10;
        dimension = 1;
    } 
    else if (getFunction().equals("FiveUnevenPeaksTrap"))
    {
        leftRange = 0;
        rightRange = 30;
        velocityMax = 15;
        dimension = 1;
    } 
    else if (getFunction().equals("Vincent1D"))
    {
        leftRange = 0.25;
        rightRange = 10;
        velocityMax = 4.875;
        dimension = 1; // can be any dimensions
    } 
    else if (getFunction().equals("Vincent2D"))
    {
        leftRange = 0.25;
        rightRange = 10;
        velocityMax = 4.875;
        dimension = 2; // can be any dimensions
    } 
    else if (getFunction().equals("Vincent3D"))
    {
        leftRange = 0.25;
        rightRange = 10;
        velocityMax = 4.875;
        dimension = 3; // can be any dimensions
    } 
    else if (getFunction().equals("Hump"))
    {
        leftRange = 0.0;
        rightRange = 1.0;
        velocityMax = 0.5;
        //dimension = 2;
    } 
  }
    
  
  public int getNumParticles() {
    return numParticles;
  }

  public void setNumParticles(int num) {
    numParticles = num;
  }

  public double getLeftRange() {
    return leftRange;
  }

  public void setLeftRange(double lr) {
    leftRange = lr;
  }
 
  public double getRightRange() {
    return rightRange;
  }

  public void setRightRange(double rr) {
    rightRange = rr;
  }

  public double getVelocityMax() {
    return velocityMax;
  }

  public void setVelocityMax(double v) {
    velocityMax = v;
  }
  
  public int getMaxIterations() {
       return maxIterations;
   }

  public void setMaxIterations(int gen)  {
       maxIterations = gen;
  }

  public int getMaxNumEvals() {
       return maxNumEvals;
   }

  public void setMaxNumEvals(int mne)  {
       maxNumEvals = mne;
  }

  public int getDimension() {
      return dimension;
  }

  public void setDimension(int dim) {
      dimension = dim;
  }
  
 
  public String getFunction() {
       return function;
   }

   public void setFunction(String f) {
       function = f;
   }
 
   public int getRun() {
       return run;
   }

   public void setRun(int r) {
       run = r;
   }
  
  public boolean getBatch() {
    return BATCH;
  }

  public void setBatch(boolean b) {
     BATCH = b;
  }


  public boolean getShuffle() {
    return SHUFFLE;
  }

  public void setShuffle(boolean sh) {
     SHUFFLE = sh;
  }

  public String getMethod() {
    return method;
  }

  public void setMethod(String m) {
     method = m;
  }
  
  public double getRandomUniform() {
    return Random.uniform.nextDoubleFromTo(0, 1);
  }

  public double getGaussian() {
    return Random.normal.nextDouble();
  }
 
 public int getcurrentpos() {
    return currentpos;
 }
 public void setcurrentpos(int c) {
    currentpos = c;
 }
 
   public double[] getnBest() {
    return nBest;
  }

  public void setNBest(double[] gb) {
    for (int i=0; i < getDimension(); i++) 
        nBest[i] = gb[i];
  }

  public double getNBestValue() {
    return nBestValue;
  }

  public void setNBestValue(double gb) {
    nBestValue = gb;
  }
 
  public double getEuDist(double[] p1, double[] p2)
  {   
    double euDist=0, sum = 0;
    for (int k=0; k < getDimension(); k++) 
         sum  +=  Math.pow(p1[k] - p2[k], 2);
    euDist = Math.sqrt(sum);
    return euDist;
  }
  
  public double getThreshold() {
    return threshold;
  }

  public void setThreshold(double th) {
    threshold = th;
  }

  public double getRadius() {
    return radius;
  }

  public void setRadius(double r) {
    radius = r;
  }

  public double getNumPeaks() {
    return numPeaks;
  }

  public void setNumPeaks(int np) {
    numPeaks = np;
  }
 
// get a list containing all different pBest according to pBestValues,(high to low).
// this is used for counting the number of peaks found
  private ArrayList getPBestList(ArrayList aList)
  {
        double[] v = new double[dimension];
        ArrayList sortedPSOList = new ArrayList();
        sortedPSOList.addAll(aList);         
        //System.out.println("sortedPSOList size "+sortedPSOList.size());
        Iterator sortedPSOListIter = sortedPSOList.iterator();    
    
        //sort particles in ascending order according to nBestValues
        try 
        {               
            int i,j;
            i = 0;
            while (i < (sortedPSOList.size() - 1))       
            {
                j = i + 1;                
                while (j < sortedPSOList.size())
                {
                    // according to nBestValues, in descending order
                    if (((PSOAgent)sortedPSOList.get(i)).getPBestValue() 
                                        < ((PSOAgent)sortedPSOList.get(j)).getPBestValue())
                    {
                        PSOAgent temp = (PSOAgent)sortedPSOList.get(i);
                        sortedPSOList.set(i, sortedPSOList.get(j));
                        sortedPSOList.set(j, temp);
                    }
                    j++;
                }
                i++;
            }
        } 
        catch (Exception e) 
        {
            e.printStackTrace (System.err);
        }   
            
        return sortedPSOList; //pBestList;
   }


/**
   * Clearing procedure to determine the species seeds in the population
   * Please refer to the species-based PSO (SPSO) (Li, 2004). 
   */
  protected ArrayList getSeeds(ArrayList aList) {
  
    boolean FOUND;
    double euDistPresent = 0.0;
    ArrayList seedList = new ArrayList();
    int seedListSize;
  
    ArrayList sortedList = new ArrayList();
    sortedList.addAll(aList);   
           
    //System.out.println("sortedList size "+sortedList.size());
    Iterator sortedIter = sortedList.iterator();    
   
    while (sortedIter.hasNext())
    {
         PSOAgent p = (PSOAgent) sortedIter.next();
         //p.PROCESSED = true;        
         FOUND = false;
                
         seedListSize = seedList.size();
         //System.out.println("current seedlist size: "+seedListSize);
         for (int i=0; i < seedListSize; i++)
         {
            PSOAgent seed = (PSOAgent)seedList.get(i);
            euDistPresent = getEuDist(p.getPBest(), seed.getPBest());            
                      
            // if Euclidean distance is less than the radius (0.5 for shubert 2D)
            if (euDistPresent < radius)  
            {
               // only useful for spso variant
               if (getMethod().equals("SPSO"))
                    p.setSeed(seed.getSeed(), seed.getSeedFit());
               
               FOUND = true;
               //System.out.println("found a similar one!"+"\n");
               break;   // break the for loop
            }
         }
         
         if (FOUND != true) 
         {
            p.setSeed(p.getPBest(), p.getPBestValue());
            seedList.add(p);
         }             
    }
    //System.out.println("current seedlist size: "+seedList.size());
    return seedList;
 }   
 

  

 // evaluate the performance  
 protected void checkAcuracy(ArrayList seedList)
 {    
    double euDist = 0.0;      
    int seedListSize = seedList.size();   

    //allPeaksFound = 0;
    //numPeaksFound = 0;
    if ((getFunction().equals("TwoPeakTrap") || (getFunction().equals("CentralTwoPeakTrap")))
            && (seedList.size() >= 1)  
            && ((200.0 - ((PSOAgent)seedList.get(0)).getPBestValue()) <= getThreshold()) )         
    {
        // to test convergence speed
        System.out.println("seed 0 :" +((PSOAgent)seedList.get(0)).getPBestValue());

        System.out.println("Acceptance Threshold :" + getThreshold());
        System.out.println("Generation :" + this.getTickCount()); 
        System.out.println("numEvals " + "\t" + numEvals);  
        allPeaksFound = 1;  // if all peaks are found!!

        this.stop();
    }
    if ((getFunction().equals("FiveUnevenPeaksTrap"))
            && (seedList.size() >= 2)  
            && ((200.0 - ((PSOAgent)seedList.get(0)).getPBestValue()) <= getThreshold())
            && ((200.0 - ((PSOAgent)seedList.get(1)).getPBestValue()) <= getThreshold()) )         
    {
        // to test convergence speed
        System.out.println("seed 0 :" +((PSOAgent)seedList.get(0)).getPBestValue());
        System.out.println("seed 1 :" +((PSOAgent)seedList.get(1)).getPBestValue());
        System.out.println("Acceptance Threshold :" + getThreshold());
        System.out.println("Generation :" + this.getTickCount()); 
        System.out.println("numEvals " + "\t" + numEvals);  
        allPeaksFound = 1;  // if all peaks are found!!

        this.stop();
    }
    if ((getFunction().equals("Deb5Even") || (getFunction().equals("Deb5Uneven")))
            && (seedList.size() >= 5)  
            && ((1.0 - ((PSOAgent)seedList.get(0)).getPBestValue()) <= getThreshold())
            && ((1.0 - ((PSOAgent)seedList.get(1)).getPBestValue()) <= getThreshold())
            && ((1.0 - ((PSOAgent)seedList.get(2)).getPBestValue()) <= getThreshold())
            && ((1.0 - ((PSOAgent)seedList.get(3)).getPBestValue()) <= getThreshold())  
            && ((1.0 - ((PSOAgent)seedList.get(4)).getPBestValue()) <= getThreshold()) )    
            //&& (this.getTickCount() > psoParameters.getMaximumGeneration()) )                  
    {
        // to test convergence speed
        System.out.println("seed 0 :" +((PSOAgent)seedList.get(0)).getPBestValue());
        System.out.println("seed 1 :" +((PSOAgent)seedList.get(1)).getPBestValue());
        System.out.println("seed 2 :" +((PSOAgent)seedList.get(2)).getPBestValue());
        System.out.println("seed 3 :" +((PSOAgent)seedList.get(3)).getPBestValue());
        System.out.println("seed 4 :" +((PSOAgent)seedList.get(4)).getPBestValue());

        System.out.println("Acceptance Threshold :" + getThreshold());
        System.out.println("Generation :" + this.getTickCount()); 
        System.out.println("numEvals " + "\t" + numEvals);  
        allPeaksFound = 1;  // if all peaks are found!!

        this.stop();
    }
    if ((getFunction().equals("Deb1Even") || (getFunction().equals("Deb1Uneven")))
            && (seedList.size() >= 1)  
            && ((1.0 - ((PSOAgent)seedList.get(0)).getPBestValue()) <= getThreshold()) )                     
    {
        // to test convergence speed
        System.out.println("seed 0 :" +((PSOAgent)seedList.get(0)).getPBestValue());

        System.out.println("Acceptance Threshold :" + getThreshold());
        System.out.println("Generation :" + this.getTickCount()); 
        System.out.println("numEvals " + "\t" + numEvals);  
        allPeaksFound = 1;  // if all peaks are found!!

        this.stop();
    }
    if ((getFunction().equals("Vincent1D"))
            && (seedList.size() >= 6)  
            && ((1.0 - ((PSOAgent)seedList.get(0)).getPBestValue()) <= getThreshold())
            && ((1.0 - ((PSOAgent)seedList.get(1)).getPBestValue()) <= getThreshold())
            && ((1.0 - ((PSOAgent)seedList.get(2)).getPBestValue()) <= getThreshold())
            && ((1.0 - ((PSOAgent)seedList.get(3)).getPBestValue()) <= getThreshold())  
            && ((1.0 - ((PSOAgent)seedList.get(4)).getPBestValue()) <= getThreshold())  
            && ((1.0 - ((PSOAgent)seedList.get(5)).getPBestValue()) <= getThreshold()) )          
    {
        // to test convergence speed
        System.out.println("seed 0 :" +((PSOAgent)seedList.get(0)).getPBestValue());
        System.out.println("seed 1 :" +((PSOAgent)seedList.get(1)).getPBestValue());
        System.out.println("seed 2 :" +((PSOAgent)seedList.get(2)).getPBestValue());
        System.out.println("seed 3 :" +((PSOAgent)seedList.get(3)).getPBestValue());
        System.out.println("seed 4 :" +((PSOAgent)seedList.get(4)).getPBestValue());
        System.out.println("seed 5 :" +((PSOAgent)seedList.get(5)).getPBestValue());

        System.out.println("Acceptance Threshold :" + getThreshold());
        System.out.println("Generation :" + this.getTickCount()); 
        System.out.println("numEvals " + "\t" + numEvals);  
        allPeaksFound = 1;  // if all peaks are found!!
        numPeaksFound = 6;  // all 6 peaks found
        
        double[] seed = new double[dimension];
        for(int i=0; i<seedList.size(); i++)
        {
            PSOAgent p = (PSOAgent)seedList.get(i);
            seed = p.getSeed();
            if (getDimension() == 1)
                System.out.println("seed "+i+","+p.getSeedFit()+","+seed[0]);
        }
        this.stop();
    }
    
    if ((getFunction().equals("Vincent2D"))
            && (seedList.size() >= 36))
    {
        boolean flag = true;
        for(int i=0; i<36; i++)  
            if ((1.0 - ((PSOAgent)seedList.get(i)).getPBestValue()) <= getThreshold() && (flag == true)) 
                flag = true;
            else
            {
                flag = false;
                break;
            }
        
        if (flag == true)
        {
            for(int i=0; i<36; i++)
                System.out.println("seed " +i+","+((PSOAgent)seedList.get(i)).getPBestValue());
  
            System.out.println("Acceptance Threshold :" + getThreshold());
            System.out.println("Generation :" + this.getTickCount()); 
            System.out.println("numEvals " + "\t" + numEvals);  
            allPeaksFound = 1;
            numPeaksFound = 36;  // all 36 peaks found
            
            this.stop();
        }
    }
    
    if ((getFunction().equals("Vincent3D"))
            && (seedList.size() >= 216))
    {
        boolean flag = true;
        for(int i=0; i<216; i++)  
            if ((1.0 - ((PSOAgent)seedList.get(i)).getPBestValue()) <= getThreshold() && (flag == true)) 
                flag = true;
            else
            {
                flag = false;
                break;
            }
        
        if (flag == true)
        {
            for(int i=0; i<216; i++)
                System.out.println("seed " +i+","+((PSOAgent)seedList.get(i)).getPBestValue());
  
            System.out.println("Acceptance Threshold :" + getThreshold());
            System.out.println("Generation :" + this.getTickCount()); 
            System.out.println("numEvals " + "\t" + numEvals);  
            allPeaksFound = 1;
            numPeaksFound = 216;  // all 216 peaks found
            
            this.stop();
        }
    }
    
    if ((getFunction().equals("SixHumpCamelBack"))
            && (seedList.size() >= 2)  
            && ((1.03163 - ((PSOAgent)seedList.get(0)).getPBestValue()) <= getThreshold())
            && ((1.03163 - ((PSOAgent)seedList.get(1)).getPBestValue()) <= getThreshold()) )    
            //&& (this.getTickCount() > psoParameters.getMaximumGeneration()) )                  
    {
        // to test convergence speed
        System.out.println("seed 0 :" +((PSOAgent)seedList.get(0)).getPBestValue());
        System.out.println("seed 1 :" +((PSOAgent)seedList.get(1)).getPBestValue());
        System.out.println("Acceptance Threshold :" + getThreshold());
        System.out.println("Generation :" + this.getTickCount()); 
        System.out.println("numEvals " + "\t" + numEvals);  
        allPeaksFound = 1;

        this.stop();
    }
    if ((getFunction().equals("Himmelblau"))
            && (seedList.size() >= 4)  
            && ((200 - ((PSOAgent)seedList.get(0)).getPBestValue()) <= getThreshold())
            && ((200 - ((PSOAgent)seedList.get(1)).getPBestValue()) <= getThreshold())
            && ((200 - ((PSOAgent)seedList.get(2)).getPBestValue()) <= getThreshold())
            && ((200 - ((PSOAgent)seedList.get(3)).getPBestValue()) <= getThreshold()) )    
            //&& (this.getTickCount() > psoParameters.getMaximumGeneration()) )                  
    {
        // to test convergence speed
        System.out.println("seed 0 :" +((PSOAgent)seedList.get(0)).getPBestValue());
        System.out.println("seed 1 :" +((PSOAgent)seedList.get(1)).getPBestValue());
        System.out.println("seed 2 :" +((PSOAgent)seedList.get(2)).getPBestValue());
        System.out.println("seed 3 :" +((PSOAgent)seedList.get(3)).getPBestValue());
        System.out.println("Acceptance Threshold :" + getThreshold());
        System.out.println("Generation :" + this.getTickCount()); 
        System.out.println("numEvals " + "\t" + numEvals); 
        allPeaksFound = 1; 

        this.stop();
    }
    if ((getFunction().equals("Foxholes"))
            && (seedList.size() >= 1)  
            && ((499.002 - ((PSOAgent)seedList.get(0)).getPBestValue()) <= getThreshold()) )                     
    {
        // to test convergence speed
        System.out.println("seed 0 :" +((PSOAgent)seedList.get(0)).getPBestValue());
        System.out.println("Acceptance Threshold :" + getThreshold());
        System.out.println("Generation :" + this.getTickCount()); 
        System.out.println("numEvals " + "\t" + numEvals); 
        allPeaksFound = 1; 

        this.stop();
    }
    if ((getFunction().equals("Shubert2D"))
            && (seedList.size() >= 18)  
            && ((186.731 - ((PSOAgent)seedList.get(0)).getPBestValue()) <= getThreshold())
            && ((186.731 - ((PSOAgent)seedList.get(1)).getPBestValue()) <= getThreshold())
            && ((186.731 - ((PSOAgent)seedList.get(2)).getPBestValue()) <= getThreshold()) 
            && ((186.731 - ((PSOAgent)seedList.get(3)).getPBestValue()) <= getThreshold())
            && ((186.731 - ((PSOAgent)seedList.get(4)).getPBestValue()) <= getThreshold())
            && ((186.731 - ((PSOAgent)seedList.get(5)).getPBestValue()) <= getThreshold())     
            && ((186.731 - ((PSOAgent)seedList.get(6)).getPBestValue()) <= getThreshold())
            && ((186.731 - ((PSOAgent)seedList.get(7)).getPBestValue()) <= getThreshold())
            && ((186.731 - ((PSOAgent)seedList.get(8)).getPBestValue()) <= getThreshold())     
            && ((186.731 - ((PSOAgent)seedList.get(9)).getPBestValue()) <= getThreshold())
            && ((186.731 - ((PSOAgent)seedList.get(10)).getPBestValue()) <= getThreshold())
            && ((186.731 - ((PSOAgent)seedList.get(11)).getPBestValue()) <= getThreshold()) 
            && ((186.731 - ((PSOAgent)seedList.get(12)).getPBestValue()) <= getThreshold())
            && ((186.731 - ((PSOAgent)seedList.get(13)).getPBestValue()) <= getThreshold())
            && ((186.731 - ((PSOAgent)seedList.get(14)).getPBestValue()) <= getThreshold())     
            && ((186.731 - ((PSOAgent)seedList.get(15)).getPBestValue()) <= getThreshold())
            && ((186.731 - ((PSOAgent)seedList.get(16)).getPBestValue()) <= getThreshold())
            && ((186.731 - ((PSOAgent)seedList.get(17)).getPBestValue()) <= getThreshold()) )                     
    {
        // to test convergence speed
        System.out.println("seed 0 :" +((PSOAgent)seedList.get(0)).getPBestValue());
        System.out.println("seed 1 :" +((PSOAgent)seedList.get(1)).getPBestValue());
        System.out.println("seed 2 :" +((PSOAgent)seedList.get(2)).getPBestValue());
        System.out.println("seed 3 :" +((PSOAgent)seedList.get(3)).getPBestValue());
        System.out.println("seed 4 :" +((PSOAgent)seedList.get(4)).getPBestValue());
        System.out.println("seed 5 :" +((PSOAgent)seedList.get(5)).getPBestValue());
        System.out.println("seed 6 :" +((PSOAgent)seedList.get(6)).getPBestValue());
        System.out.println("seed 7 :" +((PSOAgent)seedList.get(7)).getPBestValue());
        System.out.println("seed 8 :" +((PSOAgent)seedList.get(8)).getPBestValue());
        System.out.println("seed 9 :" +((PSOAgent)seedList.get(9)).getPBestValue());
        System.out.println("seed 10 :" +((PSOAgent)seedList.get(10)).getPBestValue());
        System.out.println("seed 11 :" +((PSOAgent)seedList.get(11)).getPBestValue());
        System.out.println("seed 12 :" +((PSOAgent)seedList.get(12)).getPBestValue());
        System.out.println("seed 13 :" +((PSOAgent)seedList.get(13)).getPBestValue());
        System.out.println("seed 14 :" +((PSOAgent)seedList.get(14)).getPBestValue());
        System.out.println("seed 15 :" +((PSOAgent)seedList.get(15)).getPBestValue());
        System.out.println("seed 16 :" +((PSOAgent)seedList.get(16)).getPBestValue());
        System.out.println("seed 17 :" +((PSOAgent)seedList.get(17)).getPBestValue());
  
        System.out.println("Acceptance Threshold :" + getThreshold());
        System.out.println("Generation :" + this.getTickCount()); 
        System.out.println("numEvals " + "\t" + numEvals);  
        allPeaksFound = 1;

        this.stop();
    }
    if ((getFunction().equals("Shubert3D"))
            && (seedList.size() >= 81))
    {
        boolean flag = true;
        for(int i=0; i<81; i++)  
            if ((2709.0935 - ((PSOAgent)seedList.get(i)).getPBestValue()) <= getThreshold() && (flag == true)) 
                flag = true;
            else
            {
                flag = false;
                break;
            }
        
        if (flag == true)
        {
            for(int i=0; i<81; i++)
                System.out.println("seed " +i+","+((PSOAgent)seedList.get(i)).getPBestValue());
  
            System.out.println("Acceptance Threshold :" + getThreshold());
            System.out.println("Generation :" + this.getTickCount()); 
            System.out.println("numEvals " + "\t" + numEvals);  
            allPeaksFound = 1;
    
            this.stop();
        }
    }
    if ((getFunction().equals("Shubert4D"))
            && (seedList.size() >= 324))
    {
        boolean flag = true;
        for(int i=0; i<324; i++)  
            if ((39303.55 - ((PSOAgent)seedList.get(i)).getPBestValue()) <= getThreshold() && (flag == true)) 
                flag = true;
            else
            {
                flag = false;
                break;
            }
        
        if (flag == true)
        {
            for(int i=0; i<324; i++)
                System.out.println("seed " +i+","+((PSOAgent)seedList.get(i)).getPBestValue());
  
            System.out.println("Acceptance Threshold :" + getThreshold());
            System.out.println("Generation :" + this.getTickCount()); 
            System.out.println("numEvals " + "\t" + numEvals);  
            allPeaksFound = 1;
    
            this.stop();
        }
    }
    if ((getFunction().equals("Rastrigin"))
            && (seedList.size() >= 1)  
            && ((0.0 - ((PSOAgent)seedList.get(0)).getPBestValue()) <= getThreshold()) )                     
    {
        // to test convergence speed
        System.out.println("seed 0 :" +((PSOAgent)seedList.get(0)).getPBestValue());

        System.out.println("Acceptance Threshold :" + getThreshold());
        System.out.println("Generation :" + this.getTickCount()); 
        System.out.println("numEvals " + "\t" + numEvals);  
        allPeaksFound = 1;  // if all peaks are found!!
        numPeaksFound = 1;  // the single global peak found

        this.stop();
    }
 }
 
 /** the Generic Hump func **/
protected void humpFunc()
 {
   boolean INSIDE;
 
   Peak p1 = new Peak(this);
   // Add the first peak into the peak list
   peakList.add(p1);
   while (peakList.size() < numPeaks) 
   {   
        Peak p = new Peak(this);
        INSIDE = false;
        for(int i=0; i<peakList.size(); i++)
        {
            //Peak pk = (Peak) peakIter.next();
            Peak pk = (Peak) peakList.get(i);
            double dist; 
            double sum = 0;
                       
            for(int j=0; j < dimension; j++)
                sum += Math.pow(p.location[j] - pk.location[j], 2);
            dist = Math.sqrt(sum);
            //System.out.println("pk.r: "+pk.r);  
            //System.out.println("dist: "+dist);           
            if (dist <= pk.r)
            {
                INSIDE = true;
                break;
            }

        } 
        if (!INSIDE)      
            peakList.add(p);                     
   }
 }
   
/** the number of peaks found when using the Generic Hump func **/
protected void checkHump()
{
   boolean FOUND;
   double peakRadiusThreshold;
   // iterate over all peaks 
   Iterator peakIter = peakList.iterator();  
   while (peakIter.hasNext())  
   {   
        Peak pk = (Peak) peakIter.next();
        FOUND = false;
        peakRadiusThreshold = pk.r*0.1;
        // compare with each agent in agentList
        for(int i=0; i<agentList.size(); i++)
        {
            PSOAgent p = (PSOAgent)agentList.get(i);
            double dist; 
            double sum = 0;
                       
            for(int j=0; j < dimension; j++)
                sum += Math.pow((p.getPBest())[j] - pk.location[j], 2);
            dist = Math.sqrt(sum);
            //System.out.println("pk.r: "+pk.r);  
            //System.out.println("dist: "+dist);   
                    
            if (dist <= peakRadiusThreshold)
            {
                FOUND = true;
                numPeaksFound++;
                break;
            }
        }                   
   }
   if (numPeaksFound == numPeaks)
        allPeaksFound = 1; // if all peaks are found!!
   this.stop();
}



  /**
   * Get init parameters
   * @return
   */
  public String[] getInitParam() {
    String[] params =  {"numParticles", "velocityMax", "maxIterations", "maxNumEvals", 
            "dimension", "run", "BATCH", "SHUFFLE", "function", "method", "threshold", "radius", "numPeaks"};
    return params;
  }

   public static void main (String[] args) {
    SimInit init = new SimInit();
   
    PSOModel model = new PSOModel();
    init.loadModel(model,"",BATCH);
  }

}
